//===========================================================================
//
// Animator.C
//
// By Poman So
//
//===========================================================================


#include "Animator.h"
#include "3d-tlm.h"
#include <iostream>
using namespace std;
//---------------------------------------------------------------------------
//
//	Animator Class
//
//---------------------------------------------------------------------------
static char *fieldlist[]={"Ex","Ey","Ez","Hx","Hy","Hz"};
static char *radiolist[]={"HLR"};

Animator::Animator(Window &w,char *label,Animator *A):
DTaskWindow(w,150,50,500,350,label,A),
fieldbox(*this,0,titlebar.Y2()+3,50,titlebar.Y2()+120,6,fieldlist,1,True),
radiobox(*this,fieldbox.X1(),fieldbox.Y2()+2,fieldbox.X2(),fieldbox.Y2()+20,
	 1,radiolist,0,False),
graphbox(*this,fieldbox.X2()+2,fieldbox.Y1(),Width(),Height(),100,1,0,0,0,
	 30,0,-30,True),
pushbutton(*this,PB_Margin,Height()-PB_Y1fromB,radiobox.X2()-PB_Margin,
	   Height()-PB_Margin,"Mag"){
  minW=Width();  maxW=MaxWidth();
  minH=Height(); maxH=MaxHeight();
  fb_statebits=fieldbox.StateBits();
  graphbox.SetBorder(0,0);
  graphbox.SetRubber(AllSideR);
  graphbox.SetWorld(-50,-50,50,50);
  pushbutton.SetRubber(LeftBottomR);
  connected = removeHL = False;
  mf = 1.0;
}//--- Animator::Animator ---//


Status Animator::WindowCB(CBTag tag){
  if (tag==ExposeT){
  int x=fieldbox.X2()+1;
    Line(x,0,x,Height(),Black);
  }
  else
    return DTaskWindow::WindowCB(tag);
  return Continue;
}//--- Animator::WindowCB ---//


Status Animator::ChildCB(Window *w, CBTag tag){
  if (w==&deletebutton && w==ActivatedWindow() && tag==ButtonReleaseT){
    Unmap(Watch);
  }
  else if (w==&fieldbox){
    if (tag==ActivateT){
    int fb=fieldbox.StateBits();
      if (fb!=fb_statebits){
	if (fb_statebits<=4 && fb>=8) mf*=ZO;
	if (fb_statebits>=8 && fb<=4) mf/=ZO;
	fb_statebits=fb;
	DrawGraph();
      }
    }
  }
  else if (w==&graphbox){
    switch (tag){
      case ExposeT: DrawGraph(); break;
      case DataChangeT: Parent()->ChildCB(this,DataChangeT); break;
    }
  }
  else if (w==&radiobox){
    if (tag==ActivateT){
    Boolean RHL=(radiobox.StateBits())?True:False;
      if (RHL!=removeHL){
	removeHL = RHL;
	DrawGraph();
      }
    }
  }
  else if (w==&pushbutton){
    if (ActivatedWindow()==(Window*)&pushbutton){
    static char *prompt[]={"Magnification Factor%g",""};
    int  x1=X1()+20,y1=Y1()+20,x2=x1+300,y2=y1+150;
      if (GetData(*Root(),0,x1,y1,x2,y2,"Animator: Data",prompt,&mf)==Okay){
	DrawGraph();
	Parent()->ChildCB(this,DataChangeT);
      }
    }
  }
  return Continue;
}//--- Animator::ChildCB ---//


fstream &Animator::IO(fstream &fs){
  if (connected){
    fieldbox.IO(fs);
    radiobox.IO(fs);
    if (!(fs|mf))
      cerr << "Animator::IO Error!" << endl;
    DataIO(fs,&p1,sizeof(VecU));
    DataIO(fs,&p2,sizeof(VecU));
    fb_statebits=fieldbox.StateBits();
    removeHL = (radiobox.StateBits())?True:False;
    graphbox.IO(fs);
  }
  return (Next())?Next()->IO(fs):fs;
}//--- Animator::IO ---//


void Animator::DrawGraph(){
  if (connected && Mapped() && MeshPtr_){
  float *f,*a1,*a2,cf1,cf2;
  int   i,n1,n2;
  Real  u,v,w;
  Cursor parentcursor=Parent()->SetCursor(Watch);
    MeshPtr_->GetUVW(u,v,w);
    if (p1.x==p2.x){
      n1 = p2.y-p1.y+1;  cf1 = v;
      n2 = p2.z-p1.z+1;  cf2 = w;
    }
    else if (p1.y==p2.y){
      n1 = p2.z-p1.z+1;  cf1 = w;
      n2 = p2.x-p1.x+1;  cf2 = u;
    }
    else{ //(p1.z==p2.z)
      n1 = p2.x-p1.x+1;  cf1 = u;
      n2 = p2.y-p1.y+1;  cf2 = v;
    }
    f  = new float[n1*n2];
    a1 = new float[n1];
    a2 = new float[n2];
    MeshPtr_->GetField(p1,p2,fb_statebits,f,mf);
    cf1 *= 1000;
    cf2 *= 1000;
    for (i=0;i<n1;i++) a1[i]=cf1*i;
    for (i=0;i<n2;i++) a2[i]=cf2*i;
    if (removeHL)
      graphbox.FillSurface(n1,n2,a1,a2,f,background,Black);
    else
      graphbox.Surface(n1,n2,a1,a2,f,Black);
    Flush();
    delete a2;
    delete a1;
    delete f;
    Parent()->SetCursor(parentcursor);
  }
}//--- Animator::DrawGraph ---//


void Animator::SetSamplingPlane(int *p){
  p1.x=p[0]; p1.y=p[1]; p1.z=p[2]; p2.x=p[3]; p2.y=p[4]; p2.z=p[5];
  if (p1.z==p2.z)
    graphbox.SetRotAngles(30,0,-30);
  else if (p1.y==p2.y)
    graphbox.SetRotAngles(-60,-30,-90);
  else
    graphbox.SetRotAngles(120,-60,0);
}//--- Animator::SetSamplingPlane ---//


void Animator::Init(){
  DrawGraph();
  if (Next()) Next()->Init();
}//--- Animator::Init ---//


void Animator::Update(Boolean must_refresh){
  if (visible_ && (NumIts_%Step_==0 || must_refresh==True)){
    DrawGraph();
    if (Next()) Next()->Update(must_refresh);
  }
}//--- Animator::Update ---//


//===========================================================================
//
// End of Animator.C
//
//===========================================================================
