//===========================================================================
//
// GUI.h
//
// By Poman So
//
//===========================================================================

#ifndef _GUI_h_
#define _GUI_h_

#include "Real.h"
#include "Window.h"


//---------------------------------------------------------------------------
//
//	Label Class
//
//---------------------------------------------------------------------------
class Label:public Window{
protected:
	char *label;
	void ExposeCB();
public:
	Label(Window &w, int x1, int y1, int x2, int y2, int borderwidth,
	      int threeDLook, char *s, Font f=Small, Justify hj=CenterJ,
	      Justify vj=CenterJ,Rubber rub=LeftTopR,Boolean popup=False);
	void   HighLight(Boolean rectangle=True);
	Status WindowCB(CBTag tag); 
	char   *GetLabel(){return label;}
	void   SetLabel(char *label);
};//--- Label ---//



//---------------------------------------------------------------------------
//
//	PushButton Class
//
//---------------------------------------------------------------------------
#define PB_Height  20
#define PB_Width   80
#define PB_Margin  10
#define PB_Y1fromB (PB_Height+PB_Margin)
#define PB_X1fromR (PB_Width+PB_Margin)

class PushButton:public Label{
public:
	PushButton(Window &w, int x1, int y1, int x2, int y2, char *s,
		   Font f=Small,Rubber rub=RightBottomR);
	Status WindowCB(CBTag tag);
};//--- PuahButton ---//



//---------------------------------------------------------------------------
//
//	EventButton Class
//
//---------------------------------------------------------------------------
class EventButton:public Window{
protected:
	void	ExposeCB();
public:
	EventButton(Window &w, int x1, int y1, int x2, int y2,
	             Rubber rub=DefaultRubber);
	Status	WindowCB(CBTag tag);
};//--- EventButton ---//



//---------------------------------------------------------------------------
//
//	BottomButton Class
//
//---------------------------------------------------------------------------
class BottomButton:public EventButton{
public:
	BottomButton(Window &w, int x1, int y1, int x2, int y2,
	             Rubber rub=DefaultRubber);
	Status WindowCB(CBTag tag);
};//--- BottomButton ---//



//---------------------------------------------------------------------------
//
//	ResizeButton Class
//
//---------------------------------------------------------------------------
class ResizeButton:public Window{
	Color	highlightcolor;
public:
	ResizeButton(Window &w, int x1, int y1, int x2, int y2,
		     Color highlightcolor=DefaultColor,
		     Rubber rub=DefaultRubber);
	Status WindowCB(CBTag tag);
};//--- ResizeButton ---//



//---------------------------------------------------------------------------
//
//	TitleBar Class
//
//---------------------------------------------------------------------------
#define TB_Height 16

class TitleBar:public Window{
	char         *left_title,*center_title,*right_title;
	ResizeButton *resizebutton;
	BottomButton *bottombutton;
public:
	TitleBar(Window &w, Boolean resize, Boolean bottom, char *left,
		 char *center, char *right, Font font=Small,
		 int height=TB_Height);
	~TitleBar();
	Status WindowCB(CBTag tag);
	char *LeftTitle(){return left_title;}
	void LeftTitle(char *s);
	char *CenterTitle(){return center_title;}
	void CenterTitle(char *s);
	char *RightTitle(){return right_title;}
	void RightTitle(char *s);
};//--- TitleBar ---//



//---------------------------------------------------------------------------
//
//	MenuBar Class
//
//---------------------------------------------------------------------------
#define MB_Height 16

class MenuBar:public Window{
static	int temp;
	int nextx;
public:
	MenuBar(Window &w, int y=(TB_Height+1), int height=MB_Height);
	int	ItemX1();
	int	ItemX2(char *s);
	void	SetItemX();
};//--- MenuBar ---//



//---------------------------------------------------------------------------
//
//	PulldownMenu Class
//
//---------------------------------------------------------------------------
class PulldownMenu:public Label{
static	Window	       *ActivatedMenuBar;
static	PulldownMenu   *ActivatedPDM;
static	int	       temp;
	int	       max_item_width;
	Window	       itembox;
        void	ButtonReleaseCB();
public:
	PulldownMenu(MenuBar &mb, char *name);
	Window &ItemBox(){return itembox;}
	int    ItemX1();
	int    ItemY1();
	int    ItemX2(char *s);
	int    ItemY2(char *s);
	void   SetItemXY();
	Status WindowCB(CBTag tag);
};//--- PulldownMenu ---//



//---------------------------------------------------------------------------
//
//	PulldownItem Class
//
//---------------------------------------------------------------------------
class PulldownItem:public Label{
public:
	PulldownItem(PulldownMenu &pdm, char *s);
	Status WindowCB(CBTag tag);
};//--- PulldownItem ---//



//---------------------------------------------------------------------------
//
//	MovableWindow Class
//
//---------------------------------------------------------------------------
class MovableWindow:public Window{
	Direction direction;
	int	  mx1,my1,mx2,my2;
public:
	MovableWindow(Window &w, int x1, int y1, int x2, int y2,
		Color background=DefaultColor,Color border=DefaultColor,
		Color foreground=DefaultColor,Cursor cursor=DefaultCursor,
		Font font=DefaultFont,Rubber rubber=DefaultRubber);
	void   SetMovement(Direction=DefaultDirection,
			    int mx1=0, int my1=0, int mx2=0, int my2=0);
	Status WindowCB(CBTag tag);
};//--- MovableWindow ---//




//---------------------------------------------------------------------------
//
//	ScrollBar Class
//
//---------------------------------------------------------------------------
#define SB_Margin 4
#define SB_Width  16

class ScrollBar:public Window{
public:
private:
	MovableWindow movethumb;
        Direction     direction;
	int    DecP();
	int    IncP();
	int    PageDecP();
	int    PageIncP();
        void   ExposeCB();
public:
	ScrollBar(Window &w,int x1,int y1,int x2,int y2);
	float	P1();
	float	P2();
	void	SetP2(float p2);
	void	SetP1P2(float p1, float p2, Boolean moveresize=True);
        Status	WindowCB(CBTag tag);
	Status	ChildCB(Window *w, CBTag tag);
};//--- ScrollBar ---//



//---------------------------------------------------------------------------
//
//	ListBox Class
//
//---------------------------------------------------------------------------
class ListBox:public Window{
	ScrollBar vbar;
	char      **list;
	int       num_elements,first_show,highlight_element;
	int	Margin(){return 5;}
	int	ItemHeight(){return TextHeight(list[0])+Margin();}
	int	ElementsPerPage(){
		  return (Height()-Margin())/ItemHeight();
		}
	int	MaxFirstShow(){
		  return Max(0,num_elements-ElementsPerPage());
		}
	float	P1();
	float	P2();
	void	HighLight(Boolean mark=True);
	int	SelectedElement();
	void	ShowList();
	void	DecOneLine();
	void	IncOneLine();
	void	DecOnePage();
	void	IncOnePage();
	void	ScrollList();
public:
	ListBox(Window &w,int x1,int y1,int x2,int y2,int num_elements=0,
		char **list=0, int first_show=0,int highlight_element=-1,
		Rubber=DefaultRubber);
	Status	WindowCB(CBTag tag);
	Status	ChildCB(Window *w, CBTag tag);
	fstream &IO(fstream &fs);
	void	SetList(int num_elements,char **list,int f_s=0,int hl=0);
	int	HighLighted(){return highlight_element;}
	void	HighLighted(int highlight);
};//--- ListBox ---//



//---------------------------------------------------------------------------
//
//	TextBox Class
//
//---------------------------------------------------------------------------
class TextBox:public Window{
	ScrollBar vbar;
static	char	linebuffer[81];
	Boolean handle_resize;
	char    *text;
	int     textlen,firstshow,lastshow;
	int	Margin(){return 5;}
	int	LineHeight(){return TextHeight(linebuffer)+Margin();}
	int	CharsPerLine(){
		  return (vbar.X1()-3-Margin())/TextWidth(" ")+1;
		}
	int	LinesPerPage(){return (Height()-Margin())/LineHeight();}
	float	P1();
	float	P2(Boolean findlastshow=False);
	int	FillBuffer(int start);
	void	ShowText(Boolean show=True);
	void	DecOneLine();
	Boolean	IncOneLine(Boolean showtext=True);
	void	DecOnePage();
	void	IncOnePage();
	void    ScrollText();
public:
	TextBox(Window &w, int x1, int y1, int x2, int y2, char *text=0);
	Status	WindowCB(CBTag tag);
	Status	ChildCB(Window *w, CBTag tag);
	void	SetText(char *s);
};//--- TextBox ---//



//---------------------------------------------------------------------------
//
//	TextInput Class
//
//---------------------------------------------------------------------------
class TextInput: public Window{
static	char	BS,CR,WB[2];
static  int	cursor_w,cursor_h,text_x;
	unsigned cursor_clock;
	char	*buffer;
	int	maxindex,textlen,cursor_x,cursor_y;
	Boolean	cursor_show,active;
	void	HideCursor();
	void	ShowCursor();
	void	ActivateCB();
	void	ExposeCB();
	void	NoEventCB();
	void	LastNoEventCB();
	Status	KeyPressCB();
public:
	TextInput(Window &w,int x1,int y1,int x2,int y2,int buffersize,
		  char *buffer, int borderwidth=0, Boolean active=True);
	Status	WindowCB(CBTag tag);
	void	BufferContentChangedCB();
};//--- TextInput ---//



//---------------------------------------------------------------------------
//
//	RadioBox Class
//
//---------------------------------------------------------------------------
class RadioBox: public Window{
static	int	   button_x,button_y,button_w,button_h,margin;
	Direction  direction;
	Boolean	   OnlyOneOn;
	int	   state_bits, num_buttons;// 0<num_button<17
	char	   **labels;
        void	ButtonXY(int i);
	void	DrawButton(int i);
	void	DrawLabel(int i);
	Boolean	ButtonActivated(int i);
	void	ExposeCB();
	void	ActivateCB();
public:
	RadioBox(Window &w, int x1, int y1, int x2, int y2, int num_buttons,
		 char **labels, int state_bits, Boolean OnlyOneOn=False,
		 Direction direction=Vertical);
	Status	WindowCB(CBTag tag);
	fstream &IO(fstream &fs);
	int	StateBits(){return state_bits;}
	void	StateBits(int state_b){state_bits=state_b; ExposeCB();}
	void	LabelXYs(int i, int &x1, int &y1, int &x2, int &y2);
};//--- RadioBox ---//



//---------------------------------------------------------------------------
//
//	TaskWindow Class
//
//---------------------------------------------------------------------------
class TaskWindow: public Window{
protected:
	int	 minW,minH,maxW,maxH;
	TitleBar titlebar;
        void	 ResizeCB();
public:
	TaskWindow(Window &w, int x1, int y1, int x2, int y2,
		   Boolean resizebutton, Boolean tobackbutton,
		   char *left, char *center, char *right);
	Status	WindowCB(CBTag tag);
	void	SetMinMax(int minW, int minH, int maxW, int maxH);
};//--- TaskWindow ---//



//---------------------------------------------------------------------------
//
//	MessageBox Class
//
//---------------------------------------------------------------------------
class MessageBox: public TaskWindow{
	PushButton *b1,*b2,*b3;
	TextBox	   textbox;
	char	*button_pushed;
	int	ButtonWidth(char *b_L);
	void	DeleteButtons();
	char	*ShowMessage(char *message, char *b1_L, char *b2_L,
        		     char *b3_L, Boolean beep);
public:
	MessageBox(Window &w, int x1, int y1, int x2, int y2, char *title);
	~MessageBox();
	Status	WindowCB(CBTag tag);
	Status	ChildCB(Window *w, CBTag tag);
	void	Message(char *message, char *b1_label=0,
			Boolean beep=False);
	char	*Message(char *message, char *b1_L, char *b2_L,
			Boolean beep=False);
	char	*Message(char *message, char *b1_L, char *b2_L, char *b3_L,
			Boolean beep=False);
};//--- MessageBox ---//



//---------------------------------------------------------------------------
//
//	FileBox
//
//---------------------------------------------------------------------------
#define DosFileNameSize 80
typedef char DosFileName[DosFileNameSize];
typedef int (*cmpf)(const void*,const void*);

class FileBox: public TaskWindow{
	DosFileName buffer;
	PushButton  okaybutton,cancelbutton;
	TextInput   textinput;
	ListBox	    listbox;
	int	    listsize;
	char	    **filelist,*filename;
	void	    FreeFileList();
	int	    GetFileList(char *ext);
public:
	FileBox(Window &w, int x, int y);
	~FileBox();
	Status	WindowCB(CBTag tag);
	Status	ChildCB(Window *w, CBTag tag);
	Status	GetFile(char *filename, char *ext, Boolean Open=True);
};//--- FileBox ---//



//---------------------------------------------------------------------------
//
//	DataPrompt Class
//
//---------------------------------------------------------------------------
class DataPrompt: public TaskWindow{
static	int	   prompt_x1,prompt_y1,lineheight,spacing,bsize,csize;
	PushButton okaybutton,cancelbutton;
        Window	   inputwindow;
	TextInput  **textinput;
	char	   **prompts,*buffer,*code;
	void	   **data;
	Window	   *CR_CB;
	int	   num_textinputs,textinput_x1;
	void	   ExposeCB();
	void	   GetBuffer(int i);
	void	   SetBuffer(int i);
public:
	DataPrompt(Window &w, Window *CR_CB, int x1, int y1, int x2, int y2,
		   char *title, char **prompts, Boolean *active, void **data);
	~DataPrompt();
	Status	WindowCB(CBTag tag);
	Status	ChildCB(Window *w, CBTag tag);
};//--- DataPrompt ---//



//---------------------------------------------------------------------------
//
//	GraphicBox Class
//
//---------------------------------------------------------------------------
//
//
//		World Coordinate
//
//      y                          (Xmax,Ymax)
//	---------------------------------
//	|				|
//	|				|
//	|       (xo+W*dx,yo+H*dy)	|
//	|	---------		|
//     	|	|	|		|
//	|	|	|		|
//	|	|	|H		|
//	|       |	|		|
//	|	|   W	|		|
//	|       ---------		|
//	|    (xo,yo)			|
//	|				|
//	---------------------------------x
// (Xmin,Ymin)
//
//
class GraphicBox: public Window{
	int	  leftM,topM,rightM,bottomM;
	float	  xo,yo,dx,dy,Xmin,Ymin,Xmax,Ymax;
	ScrollBar *hbar,*vbar;
	Window	  *barW;
protected:
	int  W(){return Width()-(leftM+rightM);}
	int  H(){return Height()-(topM+bottomM);}
	void WorldToWindow(float xf, float yf, int &xi, int &yi);
	void WindowToWorld(int xi, int yi, float &xf, float &yf){
	       xf = (xi-leftM)*dx+xo;
	       yf = yo-(yi+bottomM-Height())*dy;
	     }
	void WindowLength(float xf, float yf, int &xi, int &yi){
	       xi = int(xf/dx);  yi = int(yf/dy);
	     }
	void WorldLength(int xi, int yi, float &xf, float &yf){
	       xf = xi*dx;  yf = yi*dy;
	     }
	void Refresh(Direction BarToSet=DefaultDirection);
	void DecOneXUnit();
	void IncOneXUnit();
	void DecOneXPage();
	void IncOneXPage();
	void ScrollXWorld();
	void DecOneYUnit();
	void IncOneYUnit();
	void DecOneYPage();
	void IncOneYPage();
	void ScrollYWorld();
public:
	GraphicBox(Window &w,int x1,int y1,int x2,int y2,int borderwidth,
		   float xo, float yo, float dx, float dy,
		   Boolean scrollbar=False);
	~GraphicBox();
	Status WindowCB(CBTag tag);
	Status ChildCB(Window *w, CBTag tag);
	fstream &IO(fstream &fs);
	void SetMargin(int leftM,int topM,int rightM,int bottomM);
	void GetMargin(int &lM, int &tM, int &rM, int &bM){
	       lM=leftM; tM=topM; rM=rightM; bM=bottomM;
	     }
	void SetWorld(float Xmin,float Ymin,float Xmax,float Ymax,
		      Boolean move_resize_bars=True);
	void GetWorld(float &xmin, float &ymin, float &xmax, float &ymax){
	       xmin=Xmin; ymin=Ymin; xmax=Xmax; ymax=Ymax;
	     }
	void SetDrawW(float xo, float yo, float dx, float dy);
	void GetDrawW(float &Xo, float &Yo, float &Dx, float &Dy){
	       Xo=xo;  Yo=yo;  Dx=dx;  Dy=dy;
	     }
	void SetHBar(Boolean move_resize=True);
	void SetVBar(Boolean move_resize=True);
	void MapGadgets();
	void UnmapGadgets();
	void Point(float x, float y, Color co=DefaultColor);
	void Points(int n, float *xy, Color co=DefaultColor);
	void Line(float x1,float y1,float x2,float y2,Color co=DefaultColor);
	void Lines(int n, float *xy, Color co=DefaultColor);
	void FillRectangle(float x1,float y1,float x2,float y2,
			Color fillcolor=DefaultColor,
			Color drawcolor=DefaultColor);
	void Rectangle(float x1,float y1,float x2,float y2,
			Color c=DefaultColor);
	void Polygon(int n, float *polypoints, Color c=DefaultColor);
	void FillPolygon(int n, float *polypoints,
			 Color fillcolor=DefaultColor,
			 Color drawcolor=DefaultColor);
	void Ellipse(float x,float y,float xr, float yr, Color c=DefaultColor);
	void FillEllipse(float x,float y,float xr,float yr,
			 Color fillcolor=DefaultColor,
			 Color drawcolor=DefaultColor);
	void Surface(int nx,int ny,float *xy,Color c=DefaultColor);
	void FillSurface(int nx, int ny, float *xy, int *corner_depth=0,
			 Color fillcolor=DefaultColor,
			 Color drawcolor=DefaultColor);
	void OutTextXY(float x,float y, const char *s,Color c=DefaultColor);
};//--- GraphicBox ---//



//---------------------------------------------------------------------------
//
//	GraphBox Class
//
//---------------------------------------------------------------------------
#define LB_SIZE 80  /* must >= 80 */
class GraphBox: public Window{
	GraphicBox GB;
	Boolean	bargraph;
	char	title[LB_SIZE],hlabel[LB_SIZE],vlabel[LB_SIZE];
	int	num_hticks,num_vticks,num_points;
	float	*xy;
	void DrawTexts();
	void DrawHAxis();
	void DrawVAxis();
	void ExposeCB();
	void SelectCB();
	void GB_ExposeCB();
public:
	GraphBox(Window &w, int x1, int y1, int x2, int y2);
	Status WindowCB(CBTag tag);
	Status ChildCB(Window *w, CBTag tag);
	fstream &IO(fstream &fs);
	void ToBarGraph(Boolean refresh=False);
	void ToLineGraph(Boolean refresh=False);
	void Clear();
	void Line(float x1,float y1,float x2,float y2,Color c=DefaultColor){
	       GB.Line(x1,y1,x2,y2,c);
	     }
	void SetWorld(float x1,float y1,float x2,float y2,
		Boolean refresh=False);
	void GetWorld(float &x1, float &y1, float &x2, float &y2){
	       GB.GetWorld(x1,y1,x2,y2);
	     }
	void SetTitle(char *s, Boolean refresh=False);
	void GetTitle(char *s);
	void SetHLabel(char *s, Boolean refresh=False);
	void GetHLabel(char *s);
	void SetVLabel(char *s, Boolean refresh=False);
	void GetVLabel(char *s);
	void SetTicks(int num_hticks, int num_vticks, Boolean refresh=False);
	void GetTicks(int &num_ht, int &num_vt){
	       num_ht=num_hticks;  num_vt=num_vticks;
	     }
	void SetData(int num_points, float *xy, Boolean refresh=False);
	void Refresh(Boolean refreshLabels=True, Boolean refreshGraph=True);
	void DrawGraph(Boolean clearbackground=True);
	void DrawOnePoint(int i, Color c=DefaultColor);
	void AddOnePoint(float x, float y, Boolean draw_it=True);
	void SubOnePoint(Boolean erase_it=True);
};//--- GraphBox ---//



//---------------------------------------------------------------------------
//
//	DrawingBoard Class
//
//---------------------------------------------------------------------------
class DrawingBoard: public GraphicBox{
static	int   Mx,My,tx1,ty1,tx2,ty2,tw,th;
static	Button	ButtonFirstPressed;
static  Cursor oldcursor;
	float Dxo,Dyo,Dx,Dy;
	float Dx1,Dy1,Dx2,Dy2;
	void  FeedBack(Boolean showmeter=True);
	void  ActivateCB();
	void  ButtonPressCB();
	void  ButtonReleaseCB();
public:
	DrawingBoard(Window &w, int x1, int y1, int x2, int y2,
		     float xmin, float ymin, float xmax, float ymax);
	Status	WindowCB(CBTag tag);
	fstream &IO(fstream &fs);
	void	GetData(float &Dx1, float &Dy1, float &Dx2, float &Dy2);
	void	SetDrawXY(float Dxo,  float Dyo,  float Dx,  float Dy);
	void	GetDrawXY(float &Dxo, float &Dyo, float &Dx, float &Dy);
	void	DrawXY(float &x, float &y);
	void	DrawGrid();
	float	DXo(){ return Dxo; }
	float	DYo(){ return Dyo; }
};//--- DrawingBoard ---//



//---------------------------------------------------------------------------
//
//	Graphic3DBox Class
//
//---------------------------------------------------------------------------
//
//	z
//
//	|
//	|     y
//	|    /
//	|   / 	(right-handed system: x cross y is z)
//	|  /
//	| /	(rotation angles, Rx ..etc., are in degree)
//	|/
//	---------- x
//
//
//	    ||
//	    ||
//	    ||
//	    \/
//
//         (2D)
//	
//	Yp: y-projection
//
//	|
//	|
//	|
//	|
//	|
//	|
//	|
//	---------- Xp: x-projection
//
//
class Graphic3DBox: public GraphicBox{
static	float va,vb,vc,vd;
	float ProjP,Mf,Xo,Yo,Zo,Rx,Ry,Rz,Rmatrix[3][3];
	void	DataChangedCB();
	void	SelectCB();
public:
	Graphic3DBox(Window &w, int x1, int y1, int x2, int y2,float ProjP,
		     float Mf=1,float Xo=0,float Yo=0,float Zo=0,float Rx=0,
		     float Ry=0,float Rz=0,Boolean scrollbar=False);
	Status	WindowCB(CBTag tag);
	fstream &IO(fstream &fs);
	void	Projection(float ProjP);
	float	Projection(){return ProjP;}
	void	Magnification(float Mf);
	float	Magnification(){return Mf;}
	void	SetRotAngles(float Rx, float Ry, float Rz);
	void	GetRotAngles(float &rx,float &ry,float &rz){
		  rx=Rx;  ry=Ry;  rz=Rz;
		}
	void	IncRotAngles(float Rx, float Ry, float Rz);
	void	SetOrigin(float xo, float yo, float zo){
		  Xo=xo;  Yo=yo;  Zo=zo;
		}
	void	IncOrigin(float dx, float dy, float dz){
		  Xo+=dx;  Yo+=dy;  Zo+=dz;
		}
	void	RotationMatrix();
	void	Transform(float &x, float &y, float &z);
	void	Project(float x3D,float y3D,float z3D,float &x2D,float &y2D);
	void	Project(int nx,int ny,float *x,float *y,float *z,float *xy);
	void	Project(int n, float *xyz, float *xy);
	void	Box(float x1,float y1,float z1,float x2,float y2,float z2,
		     Color co=DefaultColor);
	void	Line(float x1,float y1,float z1,float x2,float y2,float z2,
		     Color co=DefaultColor);
	void	Lines(int n, float *xyz, Color co=DefaultColor);
	void	Polygon(int n, float *xyz, Color co=DefaultColor);
	void	Surface(int nx, int ny, float *x, float *y, float *z,
			Color co=DefaultColor);
	void	FillSurface(int nx, int ny, float *x, float *y, float *z,
			    Color fillcolor=DefaultColor,
			    Color drawcolor=DefaultColor);
};//--- Graphic3DBox ---//



//---------------------------------------------------------------------------
//
//	Appication Functions
//
//---------------------------------------------------------------------------
void	Message(char *message, Boolean beep=False);
void	Message(Window &w, char *message, Boolean beep=False);
char    *Message(char *message, char *b1, char *b2, Boolean beep=False);
char	*Message(Window &w, char *message, char *b1, char *b2,
		 Boolean beep=False);
char    *Message(char *message, char *b1, char *b2, char *b3,
		 Boolean beep=False);
char	*Message(Window &w, char *message, char *b1, char *b2, char *b3,
		 Boolean beep=False);
Status  GetFile(char *filename, char *ext, Boolean ToOpen=True);
Status	GetFile(Window &w, int x, int y, char *filename, char *ext,
		Boolean ToOpen=True);
Status	GetData(Window &w,Window *CR_CB, int x1,int y1,int x2,int y2,
		char *title,char **prompts,...);
Status	GetActiveData(Window &w,Window *CR_CB,int x1,int y1,int x2,int y2,
		char *title,char **prompts, Boolean *active,...);



//---------------------------------------------------------------------------
//
//	DTaskWindow Class
//
//---------------------------------------------------------------------------
class DTaskWindow: public TaskWindow{
protected:
	DTaskWindow *previous,*next;
	EventButton deletebutton;
	void	    AppendToDTaskList(DTaskWindow *DTW);
	void	    DeleteFromDTaskList();
	Boolean	    DeleteCB();
public:
	DTaskWindow(Window &w,int x1,int y1,int x2,int y2,char *center_title,
		    DTaskWindow *DTW=0);
	~DTaskWindow();
	DTaskWindow *Previous(){return previous;}
	DTaskWindow *Next(){return next;}
	DTaskWindow *LastDTaskWindow();
};//--- DTaskWindow ---//



#endif 

//===========================================================================
//
// End of GUI.h
//
//===========================================================================
