//===========================================================================
//
// Generator.C
//
// By Poman So
//
//===========================================================================

#define MAXINT 65535

#include <math.h>
#include "Generator.h"
#include "3d-tlm.h"
#include <iostream>
using namespace std;

//---------------------------------------------------------------------------
//
//	Generator Class
//
//---------------------------------------------------------------------------
float Generator::Dt;

char *fieldlist[]={"Ex","Ey","Ez","Hx","Hy","Hz"};
char *distlist[]={"Sin x","Sin y","Sin z"};
char *source = "Generator";
char *funlist[]={"Impulse",
		 "Gaussian",
		 "R Cosine",
		 "Sin",
		 "Square",
		 "Tri",
		 "NEMP",
		 "Sin  (f)",
		 "Sqr  (f)",
		 "Tri  (f)"
		};
char *prompttitle="Generator: Data";


int Generator::px1(){return X1()+20;}
int Generator::py1(){return Y1()+20;}
int Generator::px2(){return px1()+300;}
int Generator::py2(){return py1()+220;}


void Generator::DrawGraph(){
float Xmin,Ymin,Xmax,Ymax,*xy=0;
int   i,j,imax;
  if (!Mapped()) return;
  graphbox.GetWorld(Xmin,Ymin,Xmax,Ymax);
  imax=int(Min(double(Xmax),double(MAXINT/2)));
  xy = new float[2*imax];
  if (xy){
  Cursor parentcursor=Parent()->SetCursor(Watch);
    for (i=j=0;i<imax;i++){
      xy[j++]=i;
      xy[j++]=Function(i);
    }
    graphbox.SetData(imax,xy,True);
    graphbox.SetData(0,0,False);
    delete xy;
    Parent()->SetCursor(parentcursor);
  }
  else
    Message("No free memory!");
}//--- Generator::DrawGraph ---//


Status Generator::ImpulseP(){
float a=amplitude;
static char  *prompt[]={"amplitude%g",""};
  if (GetData(*Root(),0,px1(),py1(),px2(),py2(),prompttitle,prompt,
      &a)==Okay && a!=amplitude){
    amplitude=a;  return Okay;
  }
  return Cancel;
}//--- Generator::ImpulseP ---//


Status Generator::GaussianP(){
float a=amplitude,s=sigma,m=mean;
static char  *prompt[]={"amplitude%g","sigma (Dt)%g","mean (Dt)%g",""};
  if (GetData(*Root(),0,px1(),py1(),px2(),py2(),prompttitle,prompt,
	      &a,&s,&m)==Okay && (a!=amplitude || s!=sigma || m!=mean)){
    amplitude=a; sigma=s; mean=m; return Okay;
  }
  return Cancel;
}//--- Generator::GaussianP ---//


Status Generator::RCosP(){
float a=amplitude,p=period;
static char  *prompt[]={"amplitude%g","pulse length (Dt)%g",""};
  if (GetData(*Root(),0,px1(),py1(),px2(),py2(),prompttitle,prompt,
	      &a,&p)==Okay && (a!=amplitude || p!=period)){
    amplitude=a; period=p; return Okay;
  }
  return Cancel;
}//--- Generator::RCosP ---//


Status Generator::SinP(){
  return RCosP();
}//--- Generator::SinP ---//


Status Generator::SquareP(){
  return RCosP();
}//--- Generator::SquareP ---//


Status Generator::TriP(){
  return RCosP();
}//--- Generator::TriP ---//


Status Generator::NEMP(){
float a=amplitude,r=rise_t,f=fall_t;
static char  *prompt[]={"amplitude%g","rise time (Dt)%g",
			"fall time (Dt)%g",""};
  if (GetData(*Root(),0,px1(),py1(),px2(),py2(),prompttitle,prompt,
	      &a,&r,&f)==Okay && (a!=amplitude || r!=rise_t || f!=fall_t)){
    amplitude=a;  rise_t=r;  fall_t=f;  return Okay;
  }
  return Cancel;
}//--- Generator::NEMP ---//


Status Generator::Sinf(){
float  a=amplitude,f=frequency;
static char   *prompt[]={"amplitude%g","frequency [GHz]%g",""};
Status s;
  f*=1.0E-9;
  s=GetData(*Root(),0,px1(),py1(),px2(),py2(),prompttitle,prompt,&a,&f);
  f*=1.0E9;
  if (s==Okay && (a!=amplitude || f!=frequency)){
    amplitude=a; frequency=f; return Okay;
  }
  return Cancel;
}//--- Generator::Sinf ---//


Status Generator::Squaref(){
  return Sinf();
}//--- Generator::Squaref ---//


Status Generator::Trif(){
  return Sinf();
}//--- Generator::Trif ---//


void Generator::Function(){
Status s;
  switch (fb_highlighted){
    case  0: s=ImpulseP();  break;
    case  1: s=GaussianP(); break;
    case  2: s=RCosP();     break;
    case  3: s=SinP();      break;
    case  4: s=SquareP();   break;
    case  5: s=TriP();      break;
    case  6: s=NEMP();      break;
    case  7: s=Sinf();      break;
    case  8: s=Squaref();   break;
    case  9: s=Trif();      break;
    default: s=ImpulseP();
  }
  if (s==Okay) DrawGraph();
}//--- Generator::Function ---//



void Generator::ExposeCB(){
int xy[8];
  xy[0]=xy[2]=graphbox.X1()-1; xy[1]=xy[5]=graphbox.Y1();
  xy[4]=xy[6]=graphbox.X2()+1; xy[3]=xy[7]=Height();
  Lines(2,xy,border);
}//--- Generator::ExposeCB ---//


void Generator::Dt_CB(){
  if (Dt != Dt_){
    switch (fb_highlighted){
      case 7:
      case 8:
      case 9: DrawGraph(); break;
    }
    if (Next())
      Next()->Dt_CB();
    else
      Dt = Dt_;
  }
}//--- Generator::Dt_CB ---//


Generator::Generator(Window &w, char *label, Generator *G):
DTaskWindow(w,50,50,400,250,label,G),
fieldbox(*this,0,titlebar.Y2()+3,60,titlebar.Y2()+120,6,fieldlist,1,False),
distbox(*this,0,fieldbox.Y2()+3,60,fieldbox.Y2()+60,3,distlist,0,False),
function(*this,Width()-85,fieldbox.Y1(),Width(),Height(),10,funlist,0,0,
	 TopRightBottomR),
graphbox(*this,fieldbox.X2()+2,fieldbox.Y1(),function.X1()-2,Height()){
  fb_statebits = fieldbox.StateBits();
  db_statebits = distbox.StateBits();
  fb_highlighted=function.HighLighted();
  Dt = Dt_;
  graphbox.SetBorder(0,0);
  graphbox.SetRubber(AllSideR);
  graphbox.SetWorld(0,-1,100,1,False);
  graphbox.SetTitle("Generator Function");
  graphbox.SetHLabel("Time in Delta t");
  graphbox.SetVLabel("Function Amplitude");
  function.SetBorder(0,0);
  minW=Width();  maxW=MaxWidth();
  minH=Height(); maxH=MaxHeight();
  amplitude = 1;
  frequency = 1.0e9;
  period = 50;
  sigma  =  5;
  mean   = 10;
  rise_t = 5;
  fall_t = 20;
  connected = False;
}//--- Generator::Generator ---//


Status Generator::WindowCB(CBTag tag){
  switch (tag){
    case ExposeT:     ExposeCB(); break;
    case DataChangeT: Dt_CB();    break;
    default: return DTaskWindow::WindowCB(tag);
  }
  return Continue;
}//--- Generator::WindowCB ---//


Status Generator::ChildCB(Window *w, CBTag tag){
  if (w==&deletebutton && ActivatedWindow()==w && tag==ButtonReleaseT){
    Unmap(Watch);
  }
  else if (w==&fieldbox){
    if (tag==ActivateT)
      if (NumIts_==0){
	fb_statebits=fieldbox.StateBits();
	Parent()->ChildCB(this,DataChangeT);
      }
      else
	fieldbox.StateBits(fb_statebits);
  }
  else if (w==&distbox){
    if (tag==ActivateT)
      if (NumIts_==0){
	db_statebits=distbox.StateBits();
	Parent()->ChildCB(this,DataChangeT);
      }
      else
	distbox.StateBits(db_statebits);
  }
  else if (w==&graphbox){
    switch (tag){
      case DataChangeT: Parent()->ChildCB(this,DataChangeT); break;
      case SelectT:	Function();  break;
      case ExposeT:	DrawGraph(); break;
    }
  }
  else if (w==&function){
    switch (tag){
      case ButtonReleaseT:
	if (NumIts_==0){
	  fb_highlighted=function.HighLighted();
	  DrawGraph(); 
	  Parent()->ChildCB(this,DataChangeT);
	}
	else
	  function.HighLighted(fb_highlighted);
	break;
      case SelectT:
	Function();
	break;
    }
  }
  return Continue;
}//--- Generator::ChildCB ---//


fstream &Generator::IO(fstream &fs){
  if (connected){
  int bits;
    bits  = db_statebits<<6;
    bits |= fb_statebits;
    if (!(fs|amplitude|frequency|period|sigma|mean|rise_t|fall_t
            |fb_highlighted|bits))
      cerr << "Generator::IO Error!" << endl;
    fb_statebits = bits & 63;
    db_statebits = (bits>>6)&7;
    fieldbox.IO(fs);
    function.IO(fs);
    fb_statebits=fieldbox.StateBits();
    distbox.StateBits(db_statebits);
    fb_highlighted=function.HighLighted();
    graphbox.IO(fs);
  }
  return (Next())?Next()->IO(fs):fs;
}//--- Generator::IO ---//

float Generator::ImpulseP(int n){
  return (n==0)?amplitude:0;
}//--- Generator::ImpulseP ---//


float Generator::GaussianP(int n){
float v=-0.5*pow((n-mean)/sigma,2.0);
  return (n<0 || v<=-70.0)?0:amplitude*exp(v);
}//--- Generator::GaussianP ---//


float Generator::RCosP(int n){
  return (n<0 || n>period)
    ?0
    :0.5*amplitude*(1+sin((double(n)/period-0.25)*TWOPI));
}//--- Generator::RCosP ---//


float Generator::SinP(int n){
  return (n<0 || n>period)?0:amplitude*sin(TWOPI*double(n)/period);
}//--- Generator::SinP ---//


float Generator::SquareP(int n){
  return (n<0 || n>period)?0:(n<0.5*period)?amplitude:-amplitude;
}//--- Generator::SquareP ---//


float Generator::TriP(int n){
  return (n<0 || n>period)
    ?0
    :(n<0.25*period)
      ?4*amplitude*double(n)/period
      :(n<0.75*period)
	?amplitude*(2.0-4.0*double(n)/period)
	:(n<period)
	  ?4*amplitude*(double(n)/period-1)
          :0;
}//--- Generator::TriP ---//


float Generator::NEMP(int n){
double v1=-n/rise_t,v2=-n/fall_t;
  if (v1<-70.0) v1=-70.0;
  if (v2<-70.0) v2=-70.0;
  return (n<0)?0:amplitude*(exp(v2)-exp(v1));
}//--- Generator::NEMP ---//


float Generator::Sinf(int n){
  return amplitude*sin(ModTwoPi(n*Dt_*TWOPI*frequency));
}//--- Generator::Sinf ---//


float Generator::Squaref(int n){
  return ModTwoPi(n*Dt_*TWOPI*frequency)<PI?amplitude:-amplitude;
}//--- Generator::Squaref ---//


float Generator::Trif(int n){
float wt=ModTwoPi(n*Dt_*TWOPI*frequency),
      fdv1 = 0.5*PI,
      fdv2 = 3*fdv1,
      fdv3 = amplitude/fdv1,
      fdv4 = 2.0*amplitude,
      fdv5 =-4.0*amplitude;
  return (wt<fdv1)?fdv3*wt:(wt<fdv2)?fdv4-fdv3*wt:fdv5+fdv3*wt;
}//--- Generator::Trif ---//


float Generator::Function(int n){
  switch (fb_highlighted){
    case  0: return ImpulseP(n);
    case  1: return GaussianP(n);
    case  2: return RCosP(n);
    case  3: return SinP(n);
    case  4: return SquareP(n);
    case  5: return TriP(n);
    case  6: return NEMP(n);
    case  7: return Sinf(n);
    case  8: return Squaref(n);
    case  9: return Trif(n);
    default: return ImpulseP(n);
  }
}//--- Generator::Function ---//


int Generator::FieldMode(){
  return fb_statebits;
}//--- Generator::FieldMode ---//


int Generator::DistMode(){
  return db_statebits;
}//--- Generator::FieldMode ---//


//===========================================================================
//
// End of Generator.C
//
//===========================================================================
