//===========================================================================
//
// MainWind.C
//
// By Poman So
//
//===========================================================================

#include <stdio.h>
#include <math.h>
#include "Const.h"
#include "MainWind.h"
#include "Stopwatch.h"
#include <iostream>
using namespace std;

void ReportTime(const Stopwatch &watch){
char buffer[500];
  sprintf(buffer,"Elasped time is %g seconds.\n"
		 "User CPU time is %g seconds.\n"
                 "System CPU time is %g seconds.\n\n\n\n\n"
                 "Stopwatch accuracy is %g second.",
		 watch.Real(),watch.User(),watch.System(),watch.Resolution());
  Message(buffer,True);
}


//---------------------------------------------------------------------------
//
//     MainWindow Class
//
//---------------------------------------------------------------------------
char *NoFile="<No File>";

Boolean MainWindow::Go_Ahead(const char *command){
char *answer, *Yes="Yes",*No="No",message[80];
  if (FileSaved || (StkPtr=MeshEditor.StkTop())==0) return True;
  sprintf(message,"Data has not been saved; continue <%s>?",command);
  answer = Message(message,No,Yes,True);
  if (answer==Yes) FileSaved=True;
  return FileSaved;
}//--- MainWindow::GoAhead ---//


void MainWindow::DeleteTools(){
  while (GeneratorPtr[0])   delete GeneratorPtr[0];
  while (ScopePtr[0]) delete ScopePtr[0];
  while (AnimatorPtr[0]) delete AnimatorPtr[0];
  while (AnalyzerPtr[0])   delete AnalyzerPtr[0];
}//--- MainWindow::DeleteTools ---//


void MainWindow::GetStacks(){
int i,j;
  num_walls     = MeshEditor.NumElements(WALL);
  num_ErUrs     = MeshEditor.NumElements(ER_N_UR);
  num_generators   = MeshEditor.NumElements(GENERATOR);
  num_scopes = MeshEditor.NumElements(SCOPE);
  if (num_walls){
    WallStack = new Float7[num_walls];
    MeshEditor.GetWallStack(WallStack);
  }
  if (num_ErUrs){
    ErUrStack = new Float8[num_ErUrs];
    MeshEditor.GetErUrStack(ErUrStack);
  }
  if (num_generators){
    GeneratorStack = new Int8[num_generators];
    MeshEditor.GetGeneratorStack(GeneratorStack);
    for (i=0;i<num_generators;i++){
      j = GeneratorStack[i][6];
      GeneratorPtr[j]->Connect();
    }
  }
  if (num_scopes){
    ScopeStack = new Int4[num_scopes];
    MeshEditor.GetScopeStack(ScopeStack);
    for (i=0;i<num_scopes;i++){
      j = ScopeStack[i][3];
      ScopePtr[j]->SetSamplingPoint((int*)(ScopeStack[i]));
      ScopePtr[j]->Connect();
    }
  }
  GetAnimators();
}//--- MainWindow::GetStacks ---//


void MainWindow::GetAnimators(){
int i,j;
  num_animators  = MeshEditor.NumElements(ANIMATOR);
  if (num_animators){
    MeshEditor.GetAnimatorStack(AnimatorStack);
    for (i=0;i<num_animators;i++){
      j = AnimatorStack[i][6];
      AnimatorPtr[j]->SetSamplingPlane((int*)(AnimatorStack[i]));
      AnimatorPtr[j]->Connect();
    }
  }
}//--- MainWindow::GetAnimators ---//


void MainWindow::FreeStacks(){
int i;
  FreeAnimators();
  if (ScopeStack){delete ScopeStack; ScopeStack=0;  num_scopes=0;}
  if (GeneratorStack)  {delete GeneratorStack;   GeneratorStack  =0;  num_generators  =0;}
  if (ErUrStack)    {delete ErUrStack;     ErUrStack    =0;  num_ErUrs    =0;}
  if (WallStack)    {delete WallStack;     WallStack    =0;  num_walls    =0;}
  for (i=0;i<3;i++) GeneratorPtr[i]->Connect(False);
  for (i=0;i<3;i++) ScopePtr[i]->Connect(False);
  for (i=0;i<2;i++) AnalyzerPtr[i]->Connect(False);
}//--- MainWindow::FreeStacks ---//


void MainWindow::FreeAnimators(){
int i;
  num_animators=0;
  for (i=0;i<3;i++) AnimatorPtr[i]->Connect(False);
}//--- MainWindow::FreeAnimators ---//


void MainWindow::ToolDestroyCB(Window *w){
DTaskWindow *DTW=(DTaskWindow*)(w);
  if (!DTW->Previous()){
    if (DTW==GeneratorPtr[0])
      GeneratorPtr[0]=GeneratorPtr[0]->Next();
    else if (DTW==ScopePtr[0])
      ScopePtr[0]=ScopePtr[0]->Next();
    else if (DTW==AnimatorPtr[0])
      AnimatorPtr[0]=AnimatorPtr[0]->Next();
    else if (DTW==AnalyzerPtr[0])
      AnalyzerPtr[0]=AnalyzerPtr[0]->Next();
  }
}//--- MainWindow::ToolDestroyCB ---//


void MainWindow::DataChangedCB(Window *w){
  if (w==&MeshEditor){
  int i;
    for (i=0;i<3;i++){
      GeneratorPtr[i]->Unmap(Watch);
      ScopePtr[i]->Unmap(Watch);
      AnimatorPtr[i]->Unmap(Watch);
    }
    for (i=0;i<2;i++) AnalyzerPtr[i]->Unmap(Watch);
    FreeStacks();
    if (MeshPtr_){delete MeshPtr_; MeshPtr_=0;}
  }
  FileSaved = False;
}//--- MainWindow::DataChangedCB --//


void MainWindow::AnalyzerChangedCB(Analyzer *S){
Scope *R=ScopePtr[S->RefNum()],
	 *O=ScopePtr[S->OutNum()];
  S->Connect(False);
  if (R->Connected() && O->Connected()){
    S->Ref(R->Fr_Ptr(),R->Fi_Ptr());
    S->Out(O->Fr_Ptr(),O->Fi_Ptr());
    S->FreqInfo(R->Nf(),R->F1(),R->F2());
    S->SetGraph();
    S->Connect();
    S->DrawGraph();
  }
  else{
    S->Ref(0,0);
    S->Out(0,0);
  }
}//--- MainWindow::AnalyzerChangedCB ---//

void MainWindow::AnimatorChangedCB(){
int i;
  FreeAnimators();
  GetAnimators();
  for (i=0;i<3;i++)
    if (!AnimatorPtr[i]->Connected() && AnimatorPtr[i]->Mapped())
      AnimatorPtr[i]->Unmap();
  AnimatorPtr[0]->Init();
}//--- MainWindow::AnimatorChangedCB ---//


void MainWindow::InjectGenerator(Boolean forward){
    int  i, generator_num, fb_statebits, db_statebits;
    VecU ijk1, ijk2;
    VecR E, H;
    Real f;
    Dist sx, sy, sz;
    for (i=0;i<num_generators;i++){
        generator_num = GeneratorStack[i][6];
        f = GeneratorPtr[generator_num]->Function(NumIts_);
        if (!forward) f=-f;
        ijk1.x=GeneratorStack[i][0]; ijk2.x=GeneratorStack[i][3];
        ijk1.y=GeneratorStack[i][1]; ijk2.y=GeneratorStack[i][4];
        ijk1.z=GeneratorStack[i][2]; ijk2.z=GeneratorStack[i][5];
        fb_statebits = GeneratorPtr[generator_num]->FieldMode();
        db_statebits = GeneratorPtr[generator_num]->DistMode();
        if (fb_statebits &  1) E.x=f;
        if (fb_statebits &  2) E.y=f;
        if (fb_statebits &  4) E.z=f;
        if (fb_statebits &  8) H.x=f;
        if (fb_statebits & 16) H.y=f;
        if (fb_statebits & 32) H.z=f;
        sx = (db_statebits & 1)?HALF_SINE:CONSTANT;
        sy = (db_statebits & 2)?HALF_SINE:CONSTANT;
        sz = (db_statebits & 4)?HALF_SINE:CONSTANT;
        MeshPtr_->InjectEH(ijk1, ijk2, sx, sy, sz, E, H);
    }
}//--- MainWindow::InjectGenerator ---//


Boolean MainWindow::BinFileIO(int mode){
    static Real VERSION=1.0;
    static char CODE[]="3d-tlm:*.3dt";
    Real version;
    char code[sizeof(CODE)/sizeof(char)], message[80];
    
    fs.open(binfile,(mode==ios::out)?ios::out:ios::in);
    if (fs.good()){
        int    dx, dy;
        Label  *WorkInProgress;
        Cursor oldcursor;
        Boolean popup;
        if (mode==ios::out){
            iostate_ = ios::out;
            strcpy(code, CODE);
            version = VERSION;
            popup = True;
        }
        else{
            iostate_ = ios::in;
            code[0] = 0;
            version = 0;
            popup = False;
        }
        if (!(DataIO(fs, code, strlen(CODE))|version))
            cerr << "MainWindow::BinFileIO Error!" << endl;
        if (iostate_==ios::in &&
                (version!=VERSION || strncmp(code, CODE, strlen(CODE))!=0)){
            sprintf(message, "%s contains invalid data!", binfile);
            Message(message, True);
            binfile[0]=0;
            return False;
        }
        oldcursor = SetCursor(Watch);
        dx = Width()/8;  dy=Height()/16;
        WorkInProgress = new Label(*Root(), dx*3, dy*7, dx*5, dy*9, 1, 2, "", Small,
                CenterJ, CenterJ, LeftTopR, popup);
        WorkInProgress->SetLabel("Performing data I/O.");
        Flush();
        titlebar.RightTitle(binfile);
        if (!(fs|Cf_|Er_|Ur_))
            cerr << "MainWindow::BinFileIO Error!" << endl;
        MeshEditor.IO(fs);
        if (iostate_==ios::in || NumIts_==0) Si_InitCB(False);
        if (!(fs|NumIts_|StopAt_|Step_))
            cerr << "MainWindow::BinFileIO Error!" << endl;
        Flush();
        if (NumIts_>0 && MeshPtr_) MeshPtr_->IO(fs);
        Flush();
        GeneratorPtr[0]->IO(fs);
        ScopePtr[0]->IO(fs);
        AnimatorPtr[0]->IO(fs);
        AnalyzerPtr[0]->IO(fs);
        fs.close();
        FileSaved=True;
        delete WorkInProgress;
        SetCursor(oldcursor);
        return True;
    }
    else{
        fs.close();
        fs.clear();
        sprintf(message, "Unable to create/open %s", binfile);
        Message(message, True);
        binfile[0]=0;
        titlebar.RightTitle(NoFile);
        return False;
    }
}//--- MainWindow::BinFileIO ---//


void MainWindow::S_EditorCB(){
Cursor oldcursor=SetCursor(Watch);
  MeshEditor.Top();
  MeshEditor.Map();
  SetCursor(oldcursor);
}//--- MainWindow::S_EditorCB ---//


void MainWindow::S_HelpCB(){
  Message("Feature is to be implemented!");
}//--- MainWindow::S_HelpCB ---//


void MainWindow::S_AboutCB(){
static char about[]="\
3D-TLM       : Ported to CygWin-X on December 2007\n\
\n\
Contributors : Dr. Wolfgang J.R. Hoefer\n\
                     Dr. Poman P.M. So\n\
                     Computational Electromagnetics Research Laboratory\n\
                     University of Victoria\n\
\n\
References:\n\
\n\
[1] Peter B. Johns, \"A Symmetrical Condensed Node for the TLM Method\", \
IEEE Trans. Microwave Theory and Tech. vol.MTT-35, No.4, pp.370-377, \
April 1987.\n\
\n\
[2] D.A. Al-Mukhtar and J.E. Sitch, \"Transmission-line Matrix Method with \
Irregularly Graded Space\", IEE PROC. vol.128, Pt.H, No.6, pp.299-305, \
December 1981.\n\
\n\
[3] C.E. Tong and Y. Fujino, \"An Efficient Algorithm for Transmission \
Line Matrix Analysis of Electromagnetic Problems Using the Symmetrical \
Condensed Node\", IEEE Trans. Microwave Theory and Tech. vol.MTT-39,  \
No.8, pp.1420-1424, August 1991.\n\
";
int w=Width(),h=Height(),x1=w/4,y1=h/4,x2=w*3/4,y2=h*5/8;
MessageBox *AboutBox=new MessageBox(*this,x1,y1,x2,y2,"Program Information");
  AboutBox->SetMinMax(x2-x1,y2-y1,w,h);
  AboutBox->Message(about);
  delete AboutBox;
}//--- MainWindow::S_AboutCB ---//


void MainWindow::F_NewCB(){
  if (Go_Ahead("New")){
  int i;
    titlebar.RightTitle(NoFile);
    MeshEditor.SetStack(StkPtr=0,StkSize,ElementStk);
    Si_InitCB(False);
    for (i=0;i<3;i++){
      GeneratorPtr[i]->Unmap(Watch);
      ScopePtr[i]->Unmap(Watch);
      AnimatorPtr[i]->Unmap(Watch);
    }
    for (i=0;i<2;i++) AnalyzerPtr[i]->Unmap(Watch);
  }
}//--- MainWindow::F_NewCB ---//


void MainWindow::F_OpenCB(){
  if (Go_Ahead("Open")){
  char   f[DosFileNameSize];
  Status status;
    strcpy(f,binfile);
    status=GetFile(f,"3dt");
    if (status==Okay){
      F_NewCB();
      strcpy(binfile,f);
      MeshEditor.Unmap();
      BinFileIO(ios::in);
      S_EditorCB();
    }
    else if (status==Null)
      Message("No file has been selected!",True);
  }
}//--- MainWindow::F_OpenCB ---//


void MainWindow::F_SaveCB(){
  if (binfile[0]==0)
    F_SaveAsCB();
  else
    BinFileIO(ios::out);
}//--- MainWindow::F_SaveCB ---//


void MainWindow::F_SaveAsCB(){
char   f[DosFileNameSize];
Status status;
  do{
    f[0]=0;
    status = GetFile(f,"3dt",False);
    if (status==Okay){
    static char *Yes="Yes", *No="No";
    char *Answer=Yes;
    fstream fs(f,ios::in);
      if (fs){
      char message[80];
	sprintf(message,"%s already exist; overwrite it?",f);
	Answer = Message(message,No,Yes,True);
	fs.close();
      }
      if (Answer==Yes){
	titlebar.RightTitle(0);
        strcpy(binfile,f);
        BinFileIO(ios::out);
      }
      else
	status = Null;
    }
  } while (status==Null);
}//--- MainWindow::F_SaveAsCB ---//


void MainWindow::F_FreqResCB(){
fstream fs("3d-tlm.freq.rep",ios::out);
  ScopePtr[0]->FreqRes(fs);
  fs.close();
}//--- MainWindow::F_FreqResCB ---//


void MainWindow::F_TimeResCB(){
fstream fs("3d-tlm.time.rep",ios::out);
  ScopePtr[0]->TimeRes(fs);
  fs.close();
}//--- MainWindow::F_TimeResCB ---//


Status MainWindow::F_QuitCB(){
  return (Go_Ahead("Quit"))?Quit:Continue;
}//--- MainWindow::F_QuitCB ---//


void MainWindow::T_GeneratorCB(int n){
Boolean okay=False;
  if (MeshPtr_==0) okay=Si_InitCB(False);
  if (GeneratorPtr[n]->Connected()){
  Cursor oldcursor=SetCursor(Watch);
    GeneratorPtr[n]->Top();
    GeneratorPtr[n]->Map();
    SetCursor(oldcursor);
  }
  else if (!okay){
  char message[80];
    sprintf(message,"Generator[%d] is not connected!",n+1);
    Message(message,True);
  }
}//--- MainWindow::T_GeneratorCB ---//


void MainWindow::T_ScopeCB(int n){
Boolean okay=False;
  if (MeshPtr_==0) okay=Si_InitCB(False);
  if (ScopePtr[n]->Connected()){
  Cursor oldcursor=SetCursor(Watch);
    ScopePtr[n]->Top();
    ScopePtr[n]->Map();
    SetCursor(oldcursor);
  }
  else if (!okay){
  char message[80];
    sprintf(message,"Scope[%d] is not connected!",n+1);
    Message(message,True);
  }
}//--- MainWindow::T_ScopeCB ---//


void MainWindow::T_AnimatorCB(int n){
Boolean okay=False;
  if (MeshPtr_==0) okay=Si_InitCB(False);
  if (AnimatorPtr[n]->Connected()){
  Cursor oldcursor=SetCursor(Watch);
    AnimatorPtr[n]->Top();
    AnimatorPtr[n]->Map();
    SetCursor(oldcursor);
  }
  else if (!okay){
  char message[80];
    sprintf(message,"Animator[%d] is not connected!",n+1);
    Message(message,True);
  }
}//--- MainWindow::T_AnimatorCB ---//


void MainWindow::T_AnalyzerCB(int n){
  if (MeshPtr_==0) Si_InitCB(False);
  if (num_scopes>1){
  Cursor oldcursor=SetCursor(Watch);
  Analyzer *S=AnalyzerPtr[n];
     if (!S->Mapped()) AnalyzerChangedCB(S);
     S->Top();
     S->Map();
     SetCursor(oldcursor);
  }
  else{
  char message[80];
    AnalyzerPtr[n]->Connect(False);
    sprintf(message,
	"There must be at least two Scopes connected to the structure!");
    Message(message,True);
  }
}//--- MainWindow::T_AnalyzerCB ---//


void MainWindow::Si_DataCB(){
  if (ScopePtr[0]->SelectCB("Simulation Data")==Okay && NumIts_==0){
    ScopePtr[1]->CopySimData(*ScopePtr[0]);
    ScopePtr[2]->CopySimData(*ScopePtr[0]);
    AnalyzerChangedCB(AnalyzerPtr[0]);
    AnalyzerChangedCB(AnalyzerPtr[1]);
    DataChangedCB(ScopePtr[0]);
  }
}//--- MainWindow::Si_DataCB ---//


Boolean MainWindow::Si_InitCB(Boolean beep){
VecU vu;
VecR vr;
  if (MeshPtr_){ delete MeshPtr_; MeshPtr_=0; }
  FreeStacks();
  NumIts_=0;
  Xmin_=Ymin_=Zmin_=0;
  MeshEditor.MeshData(Nx_,Ny_,Nz_,Xmax_,Ymax_,Zmax_);
  vu = VecU(Nx_,Ny_,Nz_);
  vr = VecR(Xmax_,Ymax_,Zmax_);
  StkPtr = MeshEditor.StkTop();
  if (StkPtr>0 && Nx_>0 || Ny_>0 || Nz_>0){
  Cursor oldcursor=SetCursor(Watch);
    FileSaved = False;
    GetStacks();
    MeshPtr_ = new Mesh(vu,vr,Er_,Ur_);
    MeshPtr_->GetMemory();
    MeshPtr_->SetNodes(num_ErUrs,ErUrStack);
    MeshPtr_->SetBR(num_walls,WallStack);
    Dt_=MeshPtr_->Cal_Dt();
    ScopePtr[0]->Init();
    AnimatorPtr[0]->Init();
    AnalyzerPtr[0]->Init();
    SetCursor(oldcursor);
    return True;
  }
  else if (beep)
    Message("No Element has been entered!",True);
  return False;
}//--- MainWindow::Si_InitCB ---//


static char *problems="Problem Report:\n\n",
	    *nogenerator="  - no generator\n",
	    *noscope ="  - no sampling tool\n";
void MainWindow::Si_ForwardCB(){
char *errmsg;
  if (!MeshPtr_ && !Si_InitCB()) return;
  FileSaved = False;
  errmsg = new char[strlen(problems)+strlen(nogenerator)+strlen(noscope)+1];
  strcpy(errmsg,problems);
  if (num_generators==0) strcat(errmsg,nogenerator);
  if (num_scopes==0 && num_animators==0) strcat(errmsg,noscope);
  if (strlen(errmsg)>strlen(problems)){
    Message(errmsg,True);
    delete MeshPtr_;
    MeshPtr_ = 0;
    NumIts_  = 0;
    FreeStacks();
  }
  else{
  Stopwatch watch;
  char     it_count[100],*filename=titlebar.RightTitle();
  Event    event;
    if (NumIts_<0) NumIts_=0;
    GetEvent();
    watch.Start();
    if (NumIts_<StopAt_){
      visible_=True;
      while (NumIts_<StopAt_){
 	event = CheckEvents();
	if (event==ButtonRelease || event==KeyPress){
	  AnimatorPtr[0]->Update();
	  break;
	}
        titlebar.RightTitle(0);
        sprintf(it_count,"%d::%d",StopAt_,NumIts_);
        titlebar.RightTitle(it_count);
	InjectGenerator();
	ScopePtr[0]->Update();
	AnimatorPtr[0]->Update();
	AnalyzerPtr[0]->Update();
	MeshPtr_->Scatter();
	MeshPtr_->Transfer();
	MeshPtr_->Reflect();
	NumIts_++;
      }
      visible_=True;
      titlebar.RightTitle(0);
      sprintf(it_count,"%d::%d",StopAt_,NumIts_);
      titlebar.RightTitle(it_count);
      watch.Stop();
      AnimatorPtr[0]->Update();
      titlebar.RightTitle(filename);
    }
    ReportTime(watch);
  }
  delete errmsg;
}//--- MainWindow::Si_ForwardCB ---//


void MainWindow::Si_BackwardCB(){
int   stopat;
Event event;
char  *filename=titlebar.RightTitle(),it_count[100];
Stopwatch watch;
  if (NumIts_<=0){
    Message("No simulation has been performed!",True);
    return;
  }
  if (num_scopes==0 && num_animators==0){
    Message("No sampling tool has been selected!",True);
    return;
  }
  FileSaved = False;
  stopat = (NumIts_<=StopAt_)?0:StopAt_;
  watch.Start();
  if (stopat<NumIts_ && stopat>=0){
    visible_=True;
    while (NumIts_>stopat){
      event = CheckEvents();
      if (event==ButtonRelease || event==KeyPress){
	AnimatorPtr[0]->Update();
	break;
      }
      titlebar.RightTitle(0);
      sprintf(it_count,"%d::%d",StopAt_,NumIts_);
      titlebar.RightTitle(it_count);
      NumIts_--;
      MeshPtr_->Reflect(False);
      MeshPtr_->Transfer();
      MeshPtr_->Scatter();
      AnimatorPtr[0]->Update();
      ScopePtr[0]->Update(False,False);
      AnalyzerPtr[0]->Update(False);
      InjectGenerator(False);
    }
    visible_ = True;
    titlebar.RightTitle(0);
    sprintf(it_count,"%d::%d",StopAt_,NumIts_);
    titlebar.RightTitle(it_count);
    watch.Stop();
    AnimatorPtr[0]->Update();
    titlebar.RightTitle(filename);
  }
  ReportTime(watch);
}//--- MainWindow::Si_BackwardCB ---//


char *Generator1="Generator 1",
     *Generator2="Generator 2",
     *Generator3="Generator 3",
     *Scope1="Scope 1",
     *Scope2="Scope 2",
     *Scope3="Scope 3",
     *Animator1="Animator 1",
     *Animator2="Animator 2",
     *Animator3="Animator 3",
     *S11="S11",
     *S21="S21";

MainWindow::MainWindow(Display &display):
Window(display,Green,DefaultColor,DefaultColor,Arrow),
titlebar(*this,False,False,"3D-TLM","CERL @ UVic","<No file>",Small),
menubar(*this),
System(menubar,"System"),
  S_Editor(System,"Editor"),
  S_Help(System,"Help"),
  S_About(System,"About"),
File(menubar,"File"),
  F_New(File,"New"),
  F_Open(File,"Open"),
  F_Save(File,"Save"),
  F_SaveAs(File,"Save As"),
  F_FreqRes(File,"Freq Res"),
  F_TimeRes(File,"Time Res"),
  F_Quit(File,"Quit"),
T_Generator(menubar,"Generator"),
  T_Generator1(T_Generator,Generator1),
  T_Generator2(T_Generator,Generator2),
  T_Generator3(T_Generator,Generator3),
T_Scope(menubar,"Scope"),
  T_Scope1(T_Scope,Scope1),
  T_Scope2(T_Scope,Scope2),
  T_Scope3(T_Scope,Scope3),
T_Analyzer(menubar,"Analyzer"),
  T_S11(T_Analyzer,S11),
  T_S21(T_Analyzer,S21),
T_Animator(menubar,"Animator"),
  T_Animator1(T_Animator,Animator1),
  T_Animator2(T_Animator,Animator2),
  T_Animator3(T_Animator,Animator3),
Simulate(menubar,"Simulate"),
  Si_Data(Simulate,"Data"),
  Si_Init(Simulate,"Initialize"),
  Si_Forward(Simulate,"Forward"),
  Si_Backward(Simulate,"Backward"),
MeshEditor(*this){
  GeneratorPtr[0] = new Generator(*this,Generator1);
  GeneratorPtr[1] = new Generator(*this,Generator2,GeneratorPtr[0]);
  GeneratorPtr[2] = new Generator(*this,Generator3,GeneratorPtr[1]);
  ScopePtr[0]     = new Scope(*this,Scope1);
  ScopePtr[1]     = new Scope(*this,Scope2,ScopePtr[0]);
  ScopePtr[2]     = new Scope(*this,Scope3,ScopePtr[1]);
  AnalyzerPtr[0]  = new Analyzer(*this,S11,False);
  AnalyzerPtr[1]  = new Analyzer(*this,S21,True,AnalyzerPtr[0]);
  AnimatorPtr[0]  = new Animator(*this,Animator1);
  AnimatorPtr[1]  = new Animator(*this,Animator2,AnimatorPtr[0]);
  AnimatorPtr[2]  = new Animator(*this,Animator3,AnimatorPtr[1]);
  binfile[0] = 0;
  StkPtr     = 0;
  num_walls  = num_ErUrs = num_generators = num_scopes = num_animators = 0;
  StkSize    = 500;
  ElementStk = new StkElm[StkSize];
  WallStack  = 0;
  ErUrStack  = 0;
  GeneratorStack = 0;
  ScopeStack = 0;
  MeshEditor.SetStack(StkPtr,StkSize,ElementStk);
  S_EditorCB();
  FileSaved = True;
}//--- MainWindow::MainWindow ---//


MainWindow::~MainWindow(){
  if (MeshPtr_) delete MeshPtr_;
  FreeStacks();
  DeleteTools();
  delete ElementStk;
}//--- MainWindow::~MainWindow ---//


Status MainWindow::WindowCB(CBTag tag){
  return Window::WindowCB(tag);
}//--- MainWindow::WindowCB ---//


Status MainWindow::ChildCB(Window *w, CBTag tag){
  if (tag==ButtonReleaseT){
    if	    (w==&S_Editor)    S_EditorCB();
    else if (w==&S_Help)      S_HelpCB();
    else if (w==&S_About)     S_AboutCB();
    else if (w==&F_New){      F_NewCB(); S_EditorCB(); binfile[0]=0;}
    else if (w==&F_Open)      F_OpenCB();
    else if (w==&F_Save)      F_SaveCB();
    else if (w==&F_SaveAs)    F_SaveAsCB(); 
    else if (w==&F_FreqRes)   F_FreqResCB();
    else if (w==&F_TimeRes)  F_TimeResCB();
    else if (w==&F_Quit)      return F_QuitCB();
    else if (w==&T_Generator1)   T_GeneratorCB(0);
    else if (w==&T_Generator2)   T_GeneratorCB(1);
    else if (w==&T_Generator3)   T_GeneratorCB(2);
    else if (w==&T_Scope1) T_ScopeCB(0);
    else if (w==&T_Scope2) T_ScopeCB(1);
    else if (w==&T_Scope3) T_ScopeCB(2);
    else if (w==&T_Animator1) T_AnimatorCB(0);
    else if (w==&T_Animator2) T_AnimatorCB(1);
    else if (w==&T_Animator3) T_AnimatorCB(2);
    else if (w==&T_S11)       T_AnalyzerCB(0);
    else if (w==&T_S21)       T_AnalyzerCB(1);
    else if (w==&Si_Data)     Si_DataCB();
    else if (w==&Si_Init)     Si_InitCB();
    else if (w==&Si_Forward)  Si_ForwardCB();
    else if (w==&Si_Backward) Si_BackwardCB();
  }
  else if (tag==DestroyT){
    ToolDestroyCB(w);
  }
  else if (tag==DataChangeT){
    DataChangedCB(w);
  }
  else if (tag==UserDef0 && (w==AnalyzerPtr[0] || w==AnalyzerPtr[1]))
    AnalyzerChangedCB((Analyzer*)w);
  else if (tag==UserDef1 && w==&MeshEditor){
    AnimatorChangedCB();
  }
  else if (tag==UserDef2)
    Si_DataCB();
  else if (tag==UserDef3){
    ScopePtr[0]->DFT();
    AnalyzerChangedCB(AnalyzerPtr[0]);
    AnalyzerChangedCB(AnalyzerPtr[1]);
    DataChangedCB(w);
  }
  return Continue;
}//--- MainWindow::ChildCB ---//


//===========================================================================
//
// End of MainWind.C
//
//===========================================================================
