//===========================================================================
//
// Node.C
//
// By Poman So
//
//===========================================================================

#include <math.h>
#include "Node.h"


Real Node::u;
Real Node::v;
Real Node::w;

Node::Node( VecR vr, Real ier, Real iur){
  u=vr.x;  v=vr.y;  w=vr.z;  er=ier; ur=iur;
  SMatrix();
}//--- Node::Node ---//


Node::Node(){
  er=ur=1;
  SMatrix();
}//--- Node::Node ---//


Real Node::Cal_Dt(){
//
// this function should not set dt because change could not propagate
// to yx,yy,yz,zx,zy,zz & S-Matrix
//
Real m;
  m=(u>v)?u:v; m=(m>w)?(m*m):(w*w); return 0.5*u*v*w*((er<ur)?er:ur)/(m*CO);
}//--- Node::Cal_Dt ---//


void Node::SMatrix(Real idt){
Real uvw,vwu,wuv,ercdt,urcdt;
  dt=(idt>0)?idt:Cal_Dt();  ercdt=er/(CO*dt);    urcdt=ur/(CO*dt);
  uvw=2.0*u*v/w;  vwu=2.0*v*w/u;  wuv=2.0*w*u/v;
  yx=vwu*ercdt-4;  if(fabs(yx)<EPSILON) yx=0;
  yy=wuv*ercdt-4;  if(fabs(yy)<EPSILON) yy=0;
  yz=uvw*ercdt-4;  if(fabs(yz)<EPSILON) yz=0;
  zx=vwu*urcdt-4;  if(fabs(zx)<EPSILON) zx=0;
  zy=wuv*urcdt-4;  if(fabs(zy)<EPSILON) zy=0;
  zz=uvw*urcdt-4;  if(fabs(zz)<EPSILON) zz=0;
  gx=0;
  gy=0;
  gz=0;
  s1= 2*yx/(4+yx+gx);  s2= 2*yy/(4+yy+gy);  s3= 2*yz/(4+yz+gz);
  s4=-2*zx/(4+zx);     s5=-2*zy/(4+zy);     s6=-2*zz/(4+zz);
  s7= 2*gx/(4+yx+gx);  s8= 2*gy/(4+yy+gy);  s9= 2*gz/(4+yz+gz);
}//--- Node::SMatrix ---//


void Node::SetErUr(Real ier, Real iur){
  er=ier;  ur=iur;  SMatrix();
}//--- Node::SetErUr ---//


//===========================================================================
//
// End of Node.C
//
//===========================================================================
