//===========================================================================
//
// Window.h
//
// By Poman So
//
//===========================================================================

#ifndef _Window_h_
#define _Window_h_

#include "Real.h"

enum Button{LeftB,RightB,BothB};

enum Color{DefaultColor,Black,Blue,Green,Cyan,Red,Magenta,Brown,
	   DarkGray,LightGray,LightBlue,LightGreen,LightCyan,LightRed,
	   LightMagenta,Yellow,White};

enum Cursor{DefaultCursor,Arrow,Cross,Hand,Pencil,Watch};

enum Direction{DefaultDirection,Horizontal,Vertical};

enum FillStyle{DefaultFillStyle,EmptyF,SolidF,LineF,LTSlashF,
	       SlashF,BKSlashF,LTBKSlashF,HatchF,XHatchF,InterLeaveF,
	       WideDotF,CloseDotF};

enum Font{DefaultFont,Gothic,SansSerif,Small,Standard,Triplex};

enum Justify{DefaultJustify,LeftJ,TopJ,RightJ,BottomJ,CenterJ};

enum LineStyle{DefaultLineStyle,SolidL,DottedL,CenterL,DashedL};

enum LineWidth{DefaultLineWidth,Normal,Thick};

enum WriteMode{DefaultWriteMode,SetMode,XorMode};

enum Event{NoEvent,Activated,ButtonPress,ButtonRelease,KeyPress,
	   KeyRelease,Enter,Leave,Expose,MapE,UnmapE,MoveE,MotionE,
	   ResizeE,MoveResizeE,ConfigureE,PropertyE,UnknownE};

enum Rubber{DefaultRubber,LeftR,TopR,LeftTopR,RightR,LeftRightR,
	    TopRightR,LeftTopRightR,BottomR,LeftBottomR,TopBottomR,
	    LeftTopBottomR,RightBottomR,LeftRightBottomR,
	    TopRightBottomR,AllSideR};

enum Status{Quit,Continue,Okay,Cancel,Select,Null};

enum CBTag{ActivateT,SelectT,ButtonPressT,ButtonReleaseT,KeyPressT,
	   DeActivateT,EnterT,LeaveT,ExposeT,MapT,UnmapT,MoveT,
	   DResizeT,ResizeT,NoEventT,LastNoEventT,DecT,IncT,PageDecT,
	   PageIncT,ScrollT,HDecT,HIncT,HPageDecT,HPageIncT,HScrollT,
	   VDecT,VIncT,VPageDecT,VPageIncT,VScrollT,CreateT,DestroyT,
	   DataChangeT,PropertyT,UserDef0,UserDef1,UserDef2,UserDef3,
	   UserDef4,UserDef5,UserDef6,UserDef7,UserDef8,UserDef9};


//---------------------------------------------------------------------------
//
//	Display Class
//
//---------------------------------------------------------------------------
class Display{
//
//	--- Data ---
//
	char *x_root_window,**argv;
	int  argc,width,height;
public:
//
//	--- Function ---
//
	Display(int argc, char **argv);
virtual	~Display();
	int  Width(){return width;}
	int  Height(){return height;}
	int  Argc(){return argc;}
	char **Argv(){return argv;}
	char *Root(){return x_root_window;}
	void Flush();
	void Sync(Boolean discard=False);
};//--- Display ---//


//---------------------------------------------------------------------------
//
//	Window Class
//
//---------------------------------------------------------------------------
class Window{
static	Window	   *ActivatedWindow_,*EnteredWindow_,*ExposedWindow_,
		   *NextWindow_,*NoEventWindow_,*DispatchingWindow_;
static	Event	   WindowEvent_;
static	unsigned   EventClock_;
static	int	   clip_,maxx_,maxy_,MouseX_,MouseY_,KeyPressed_;
static	Button	   MouseButton_;
static	Color	   drawcolor_,fillcolor_,drawC_,fillC_;
static	Cursor	   cursor_;
static	FillStyle  fillstyle_;
static	Font	   textfont_;
static	Direction  textdir_;
static	Justify	   hjustify_,vjustify_;
static	int	   charsize_;
static	LineStyle  linestyle_;
static	LineWidth  linewidth_;
static	WriteMode  writemode_;
static	Boolean	   monochrome_;
	char	   *x_window;
	Window	   *parent,*Sibling,*sibling,*Child,*child;
	int	   x1,y1,x2,y2,rm,bm,borderwidth,threeDLook;
protected:
//
//	--- Protected Data ---
//
static	Boolean	  visible_;
	Color	  background,border,drawcolor,fillcolor;
	Cursor	  cursor;
	FillStyle fillstyle;
	Font	  textfont;
	Direction textdir;
	Justify	  hjustify,vjustify;
        int	  charsize;
	Rubber	  rubber;
	LineStyle linestyle;
	LineWidth linewidth;
	WriteMode writemode;
	Boolean	  mapped,map_requested,popup;
//
//	--- Protected Functions ---
//
	int     ColorNum(Color c);
	int     DrawColorNum(Color c=DefaultColor);
	int     FillColorNum(Color c=DefaultColor);
	int	FillStyleNum();
	int	LineStyleNum();
	int	LineWidthNum();
	int	HjustifyNum();
        int	VjustifyNum();
	int	TextFontNum(Font textfont=DefaultFont);
	int	TextDirNum();
	int	WriteModeNum();
public:
static	void	Initialize();
//
//	--- Constructors & Destructor ---
//
	Window(Display &parent, Color background=DefaultColor,
	       Color border=DefaultColor, Color foreground=DefaultColor,
	       Cursor cursor=DefaultCursor, Font font=DefaultFont,
	       int borderwidth=1);
	Window(Window &parent, int x1, int y1, int x2, int y2,
	       Color background=DefaultColor,Color border=DefaultColor,
	       Color foreground=DefaultColor,Cursor cursor=DefaultCursor,
	       Font font=DefaultFont,Rubber rubber=DefaultRubber,
	       int borderwidth=1);
virtual	~Window();
//
//	--- Window Information Functions ---
//
static	void	MonoChrome(Boolean mono=True){monochrome_=mono;}
static  Boolean Mono(){return monochrome_;}
static	Window	*ActivatedWindow(){return ActivatedWindow_;}
static	Window	*DispatchingWindow(){return DispatchingWindow_;}
	Window	*Parent(){return parent;}
	Window	*Root();
static  float	TimeSec(unsigned c_start, unsigned c_end);
static	unsigned EventClock(){return EventClock_;}
static	int	KeyPressed(){return KeyPressed_;}
static	int	MaxWidth(){return maxx_;}
static	int	MaxHeight(){return maxy_;}
	void	Position(int &X1, int &Y1, int &X2, int &Y2){
		  X1=x1;  Y1=y1;  X2=x2;  Y2=y2;
		}
	Button	MouseButton(){return MouseButton_;}
	void	MouseAt(int &x, int &y);
	void	MouseXY(int &x, int &y){
		  x=MouseX_; y=MouseY_;
		}
	int	MouseX(){return MouseX_;}
	int	MouseY(){return MouseY_;}
	int	X1(){return x1;}
	int	Y1(){return y1;}
	int	X2(){return x2;}
	int	Y2(){return y2;}
	int	Width(){return x2-x1;}
	int	Height(){return y2-y1;}
	int	ThreeDLook(){return threeDLook;}
	int	BorderWidth(){return borderwidth;}
	Boolean	Mapped(){return mapped;}
//
//	--- Window Manipulation Functions ---
//
	void	Expose();
	void	RePaint();
	void	InsertToParent();
	void	DeleteFromParent();
	void	Map();
	void	Unmap(Boolean map_requested=False);
	void	Unmap(Cursor wait_cursor);
	void	Top();
	void	Bottom();
	void	Move(int X1, int Y1, Boolean Top=False);
	void	Resize(int X2, int Y2, Boolean Top=False);
	void	MoveResize(int X1,int Y1,int X2,int Y2,Boolean Top=False);
	void	Popup(Boolean B){popup=B;}
	void	SetBorder(int borderwidth, int threeDLook);
	void	SetX1Y1X2Y2(int x1, int y1, int x2, int y2);
	Cursor	SetCursor(Cursor cursor=DefaultCursor);
	void	SetRubber(Rubber rub){rubber=rub;}
	void	ToNoEventWindow();
	void	DragMove(Direction movement= DefaultDirection,
			 int mx1=0, int my1=0, int mx2=0, int my2=0);
	void	DragResize(int minW, int minH, int maxW, int maxH);
	void	GrabPointer(int ower_event=1);
	void	UngrabPointer();
//
//	--- Graphics Functions ---
//
	void	Flush();
	void	Sync(Boolean discard=False);
virtual	Boolean	SaveBackground();
virtual	Boolean RestoreBackground();
	Boolean SaveImage(int x1, int y1, int x2, int y2);
	Boolean RestoreImage(int x, int y);
static	void	Bell(int percent=0);
	void	Clear();
	void	Clear(int x1, int y1, int x2, int y2);
	void	SetWriteMode(WriteMode wm);
	void	SetLineStyle(LineStyle ls, LineWidth lw=Normal);
	void	SetFillStyle(FillStyle fs, Color fc);
	void	DrawOverSW(Boolean mode=True);
	void	Point(int x, int y, Color c=DefaultColor);
	void	Points(int n, int *xy, Color c=DefaultColor);
	void	Line(int x1, int y1, int x2, int y2, Color c=DefaultColor);
	void	Lines(int n, int *nlines, Color c=DefaultColor);
	void	Rectangle(int x1, int y1, int x2, int y2,
			  Color c=DefaultColor);
	void	FillRectangle(int x1, int y1, int x2, int y2,
			  Color fillcolor=DefaultColor,
			  Color drawcolor=DefaultColor);
	void	Polygon(int n, int *polypoints, Color c=DefaultColor);
	void	FillPolygon(int n, int *polypoints,
			    Color fillcolor=DefaultColor,
			    Color drawcolor=DefaultColor);
	void	Ellipse(int x, int y, int xradius, int yradius,
			    Color c=DefaultColor);
	void	FillEllipse(int x, int y, int xradius, int yradius,
			    Color fillcolor=DefaultColor,
			    Color drawcolor=DefaultColor);
	void	Surface(int nx, int ny, int *xy, Color c=DefaultColor);
	void	FillSurface(int nx, int ny, int *xy, int *d,
			    Color fillcolor=DefaultColor,
			    Color drawcolor=DefaultColor);
	void	SetTextStyle(Font f=DefaultFont,
			     Direction d=DefaultDirection, int size=0);
	void	SetTextJustify(Justify h=LeftJ, Justify v=BottomJ);
	void	OutTextXY(int x,int y, const char *s,Color c=DefaultColor);
	int	TextWidth(const char *s, Font ft=DefaultFont);
	int	TextHeight(const char *s, Font ft=DefaultFont);
//
//	--- Event Handling Functions ---
//
	Event	GetEvent();
	Event	CheckEvents();
	Window	*IdentifyEventWindow(Event event=UnknownE);
	Status	DispatchEvents();
	Window  *Wait(Event event);
	void	ConfigureCB();
virtual	Status	WindowCB(CBTag tag);		// return Continue;
virtual Status  ChildCB(Window *w, CBTag tag);	// return Continue;
virtual fstream &IO(fstream &fs);		// return fs;
};//--- Window ---//

#endif

//===========================================================================
//
// End of Window.h
//
//===========================================================================
